@@ -589,6 +589,44 @@ e_mapi_util_recip_entryid_decode (EMapiConnection *conn, const struct Binary_r *
 	return FALSE;
 }
 
+gboolean
+e_mapi_util_recip_entryid_decode_dn (const struct SBinary_short *entryid,
+				     gchar **exchange_dn)
+{
+	struct Binary_r ei;
+
+	if (!entryid)
+		return FALSE;
+
+	ei.cb = entryid->cb;
+	ei.lpb = entryid->lpb;
+
+	return recip_entryid_decode_ex (&ei, exchange_dn);
+}
+
+gboolean
+e_mapi_util_recip_entryid_equal (const struct SBinary_short *entryid1,
+				 const struct SBinary_short *entryid2)
+{
+	gchar *dn1 = NULL, *dn2 = NULL;
+	gboolean same = FALSE;
+
+	if (!entryid1 && !entryid2)
+		return TRUE;
+
+	if (!entryid1 || !entryid2 || !entryid1->lpb || !entryid2->lpb || entryid1->cb != entryid2->cb)
+		return FALSE;
+
+	same = e_mapi_util_recip_entryid_decode_dn (entryid1, &dn1) &&
+	       e_mapi_util_recip_entryid_decode_dn (entryid2, &dn2) &&
+	       dn1 && dn2 && g_ascii_strcasecmp (dn1, dn2) == 0;
+
+	g_free (dn1);
+	g_free (dn2);
+
+	return same;
+}
+
 /**
  * e_mapi_util_profiledata_from_settings:
  * @empd: destination for profile settings
@@ -1213,3 +1251,31 @@ e_mapi_utils_copy_to_mapi_SPropValue (TALLOC_CTX *mem_ctx,
 
 	return FALSE;
 }
+
+static gpointer
+unref_object_in_thread (gpointer ptr)
+{
+	GObject *object = ptr;
+
+	g_return_val_if_fail (object != NULL, NULL);
+
+	g_object_unref (object);
+
+	return NULL;
+}
+
+void
+e_mapi_utils_unref_in_thread (GObject *object)
+{
+	GError *error = NULL;
+
+	if (!object)
+		return;
+
+	g_return_if_fail (G_IS_OBJECT (object));
+
+	if (!g_thread_create (unref_object_in_thread, object, FALSE, &error)) {
+		g_warning ("%s: Failed to run thread: %s", G_STRFUNC, error ? error->message : "Unknown error");
+		g_object_unref (object);
+	}
+}
